from Lexeme import * 

class LexemeLexError(Lexeme):
    def __init__(self, text, receivedChar, expectedDescription):
        Lexeme.__init__(self, text)
        self.receivedChar = receivedChar
        self.expectedDescription = expectedDescription
        
    def __str__(self):
        return "[LexError! Received : '" + self.receivedChar + "', expected: "+expectedDescription+ " in: " + text + "]"
 
#------------------------------------------------------------------------------------------------------
class D600PhoneLexer:
    whitespace = set(['\n','\r','\t',' '])
    wordDelimiters = whitespace | set([',','"',None])
    STX = '\02'
    ETX = '\03'
    cache = []

    def __init__(self, phoneConnection):
        self.pc = phoneConnection
        self.nextCh = ''
        self.cache = []

    def readToken(self, skipLeadingWhitespace = False):
        token = self.readTokenIncludingWhitespace()
#        print "readToken: ",token
        if skipLeadingWhitespace:
            while isinstance(token, LexemeWhitespace):
                token = self.readTokenIncludingWhitespace()
#                print "readToken2: ",token                
        return token

    def unreadToken(self, token):
        self.cache.insert(0,token)

    def readTokenIncludingWhitespace(self):
        if self.cache:
            return self.cache.pop()
        firstCh = self.pc.readByte()
        if firstCh == None:
#            print "readTokenIncludingWhitespace: TimeOut from connection"
            return None # Pass timeouts up.
        if firstCh == ',':
            return LexemeComma(firstCh)
        elif firstCh in self.whitespace:
            return LexemeWhitespace(firstCh)
        elif firstCh == '"':
            return self.quotedStringChar2()
        else:
            return self.wordChar2(firstCh)

    def readPascalStringPart2(self, length):
        """Put lexer in special mode to read the second-half of a Pascal string."""
        return LexemePascalString(self.pc.readBytes(length))

    def wordChar2(self,firstCh):
        text = firstCh
        
        nextCh = self.pc.readByte()
        while nextCh not in self.wordDelimiters:
            text += nextCh
            nextCh = self.pc.readByte()            
        self.pc.unreadByte(nextCh)

        if (text == "OK"
            or text == "#OK#"): # Appears this way in file listing..
            return LexemeOK(text)
        elif text == "ERROR":
            return LexemeError(text)

        return LexemeWord(text)

    def quotedStringChar2(self):
        nextCh = self.pc.readByte()
        if nextCh == self.STX:
            return self.STXQuotedStringChar3()
        
        text = ''
        while not nextCh == '"':
            text += nextCh
            nextCh = self.pc.readByte()
            if nextCh == None: # Timeout
                return LexemeLexError(text,'timeout','end-quotes')

        return LexemeQuotedString(text)

    def STXQuotedStringChar3(self):
        text = ''

        nextCh = self.pc.readByte()
        while not nextCh == self.ETX:
            text += nextCh
            nextCh = self.pc.readByte()
            if nextCh == None: # Timeout
                return LexemeLexError(text,'timeout','ETX')
            
        lastQuote = self.pc.readByte()
        if lastQuote <> '"':
            return LexemeLexError(text,lastQuote,'closing quotes after ETX')

        return LexemeSTXQuotedString(text)

