from D600ParserLog import * 
class PhoneConnection():
    """Abstract class representing a serial-like connection to a phone,
       with a cache."""
    
    def getByte(self):
        """Returns the next byte of the stream, or None if there is
           a (moderately long) timeout."""
        raise NotImplemented

    def unreadByte(self, byte):
        """Push some a byte back into the cache to be read again as part of the stream.
        Pushing a Timeout (None) has no effect."""

    def write(self, bytes):
        """Writes bytes out to the stream"""
        raise NotImplemented

    def close(self):
        """Closes theconnection for cleanup"""
        raise NotImplemented

import serial
import string
class SerialPhoneConnection(PhoneConnection):

    def __init__(self):
        # TO DO: Com Port discovery
        self.ser = serial.Serial('Com3',19200,timeout=5)
        self.cache = []

    def readByte(self):
        assert(self.ser.isOpen())
        if len(self.cache) == 0:
            thisByte = self.ser.read(1)
            if not thisByte:
                # Still nothing
#                print "readByte is returning TimeOut"
                return None
            self.cache = str(thisByte)
#            connectionlogger.debug("Reading "+str(self.cache))

        result = self.cache[0]
        self.cache = self.cache[1:]
        return result

    def readBytes(self, length):
        """Used in special lexer mode where the length of a string is known."""
        assert(self.ser.isOpen())
        if len(self.cache) < length:
            extraBytes = self.ser.read(length - len(self.cache))
            self.cache += str(extraBytes)
        result = string.join(self.cache[:length],'')
        self.cache = self.cache[length:]
#        print "READ BYTES: ", result
        return result

    def unreadByte(self, byte):
        if byte:
           self.cache = str(byte) + self.cache

    def write(self, bytes):
        assert(self.ser.isOpen())        
        result = self.ser.write(bytes)
#        print "Write:", bytes

    def close(self):
        self.ser.close()
