import MessageBuilder
import CommandProtocol
class PhoneModes:
    """The phone has a number of different settings that each have several modes.
        E.g.
            Echo can be On or Off
            The PhoneBook can be in Phone Memory, SIM Card or other modes.
            The MessageBook can be in Phone Memory, SIM Card or other modes.
        This class tracks the modes of the phone, and allows the CommandProtocol object
        to demand a particular mode be set, as required.

        While it isn't a problem in practice, be aware that there is a circular
        dependency between this and the CommandProtocol"""
    
    def __init__(self, commandProtocol):
        self.modes = {}
        self.cp = commandProtocol

    def ensureEchoOff(self):
        if self.modes.get('Echo') <> 'Off':
            self.cp.setEcho(False)
            self.modes['Echo'] = 'Off'

    def ensurePhoneBookMode(self, mode):
        assert(mode in MessageBuilder.legalPhoneBookModes), "Bad Phone Book mode "+str(mode)
        if self.modes.get('PhoneBook') <> mode: 
            self.ensureEchoOff()
            self.cp.setPhoneBookMode(mode)
            self.modes['PhoneBook'] = mode

    def ensureMessageBookMode(self, mode):
        assert(mode in MessageBuilder.legalMessageBookModes), "Bad Message Book mode "+str(mode)
        if self.modes.get('PhoneBook') <> mode:
            self.ensureEchoOff()
            self.cp.setMessageBookMode(mode)
            self.modes['MessageBook'] = mode            

