import PhoneDOM
import PhoneConnection
import CommandProtocol
import Birthday
import datetime
import ProgressBar
from D600ParserLog import * 

pbar = None
progressBarInstanceCount = 0
def callback(done, total, comment):
    global pbar
    global progressBarInstanceCount
    if pbar == None:
        progressBarInstanceCount += 1
        pbar = ProgressBar.progressbarClass(total,str(progressBarInstanceCount)[-1])
    pbar.progress(done)
def resetProgressBar():
    global pbar
    pbar = None

def cleanText(s):
    result = ''
    for ch in s:
        if ch not in "\r\n": # I'm sure there is more, but the character set isn't clear yet.
            result += ch
    return result

def isInteresting(contact):
    return (isinstance(contact, PhoneDOM.Contact_PhoneMemory)
            and (
                contact.cellNumber
                or contact.homeNumber
                or contact.officeNumber
                or contact.faxNumber
                or contact.otherNumber
                )
            )

def findEquivalent(thisItem, phoneItemDictionary):
    for key, item in phoneItemDictionary.iteritems():
        if thisItem.isEquivalent(item):
            return key
    return None

accessSecuredData = False

import win32com.client
import pywintypes

def fetchBirthdaysContactsFromOutlook():
    o = win32com.client.Dispatch("Outlook.Application")
    ns = o.GetNamespace("MAPI")
    profile = ns.Folders.Item("JulianC - Outlook1")
    outlookContacts = profile.Folders.Item("PostWork Master Contacts")

    bdayResults = set([])
    contacts = PhoneDOM.PhoneItemDict()

    resetProgressBar()
    for itemNumber in range(0,outlookContacts.Items.Count):
        outlookContact = outlookContacts.Items[itemNumber]

        if outlookContact.birthday.Format() <> "01/01/01 00:00:00":
            birthdatetime = datetime.datetime.strptime(outlookContact.birthday.Format(),"%m/%d/%y %H:%M:%S")
            birthdate = datetime.date(birthdatetime.year,birthdatetime.month,birthdatetime.day) # strip time
            if birthdate > datetime.date.today():
                # Two digit years mean confused centuries.
                birthdate = datetime.date(birthdate.year-100, birthdate.month, birthdate.day)
            thisBirthday = Birthday.Birthday(outlookContact.FirstName, outlookContact.LastName, birthdate)
            bdayResults.add(thisBirthday)

        emailAddress = '' if (not accessSecuredData) or outlookContact.Email1AddressType <> 'SMTP' else outlookContact.Email1Address
        notes = '' if (not accessSecuredData) else cleanText(outlookContact.Body)

        firstName = outlookContact.firstName
        if firstName == None or firstName == '':
            firstName = outlookContact.title
            

        thisContact = PhoneDOM.Contact_PhoneMemory(
            None,
            firstName,
            outlookContact.LastName,
            outlookContact.MobileTelephoneNumber,
            outlookContact.HomeTelephoneNumber,
            outlookContact.BusinessTelephoneNumber,
            outlookContact.BusinessFaxNumber, # Should be smarter
            outlookContact.OtherTelephoneNumber, # Should be smarter
            emailAddress,
            notes
            )

        if isInteresting(thisContact):
            contacts.add(thisContact)

        callback(itemNumber + 1,outlookContacts.Items.Count,"Contact read")
            
    o = None
    ns = None

    return bdayResults, contacts

def fetchEventsFromOutlook(maxRecords):

    # Events should go back at least this far, and forward at least this far (subject to space)
    eventWindow = {
        'start' : datetime.datetime.today() - datetime.timedelta(4*7),
        'end' : datetime.datetime.today() + datetime.timedelta(365 * 2)
        }
    olFolderCalendar = 9

    o = win32com.client.Dispatch("Outlook.Application")
    ns = o.GetNamespace("MAPI")
    profile = ns.Folders.Item("JulianC - Outlook1")

    calendar = o.Session.GetDefaultFolder(olFolderCalendar)
    items = calendar.items
    items.IncludeRecurrences = True
    items.sort("[Start]")

    restriction = ("[End] >= '"
                   +str(eventWindow['start'].day)+"/"+str(eventWindow['start'].month)+"/"+str(eventWindow['start'].year)+" 00:00 AM'"
                   + " and [Start] <= '"
                   +str(eventWindow['end'].day)+"/"+str(eventWindow['end'].month)+"/"+str(eventWindow['end'].year)+" 00:00 AM'"
                   )
    ri = items.Restrict(restriction)
    ri.Sort("[Start]")

#    maxRoom = 400-40-108
    count = 0
    skippedRecords = 0

    result = PhoneDOM.PhoneItemDict()

    finalDateTime = None

    resetProgressBar()
    maxEvents = 1000 # min(ri.Count,maxRecords)
    
    for appt in ri:
        alarmSettings = None
        if appt.ReminderSet:
            # Used to also check appt.ReminderPlaySound, but it is False even when there is a sound to play.
            alarmSettings = PhoneDOM.AlarmSettings.minutesFactory(appt.ReminderMinutesBeforeStart,appt.AllDayEvent)

        startDateTime = datetime.datetime(
            appt.Start.year,
            appt.Start.month,
            appt.Start.day,
            appt.Start.hour,
            appt.Start.minute
            )


        if appt.AllDayEvent:
            if appt.Subject[-8:] == "Birthday":
                # Special handling of birthdays; don't need default Outlook behaviour
                skippedRecords += 1
                continue

            thisEvent = PhoneDOM.Anniversary(
                eventId = None,
                text = appt.Subject,
                startDateTime = startDateTime,
                alarmSettings = alarmSettings,
                annualRecurrence = 0 # Never recur; It'll be taken care of.
                # Being smarter than this requires two changes:
                #   Working out when there are no exceptions.
                #   Marking this event so it doesn't get added as recurring again next year.
                )
        else:
            endDateTime = datetime.datetime(
                appt.End.year,
                appt.End.month,
                appt.End.day,
                appt.End.hour,
                appt.End.minute
                )
                
            thisEvent = PhoneDOM.Schedule(
                itemNumber = None,
                text = '' if (not accessSecuredData) else cleanText(outlookContact.Body),
                startDateTime = startDateTime,
                subject = appt.Subject,
                endDate = endDateTime,
                eventLocation = appt.Location,
                alarmSettings = alarmSettings,
                recurrenceSettings = None # Never recur; It'll be taken care of.):
                )

        result.add(thisEvent)                

        finalDateTime = datetime.datetime(appt.Start.year, appt.Start.month, appt.Start.day,0,0)
       
        count = count + 1
        callback(count+skippedRecords,maxEvents,"Read event")
        if count >= maxRecords:
            break
    if finalDateTime:
        result.add(
            PhoneDOM.Anniversary(
                eventId = None,
                text = "Warning! Re-sync phone to get appointments past this point.",
                startDateTime = finalDateTime 
                )
            )

    o = None
    ns = None

    return result

     
applogger.info("Starting serial connection.")
pc = PhoneConnection.SerialPhoneConnection()
applogger.info("Started serial connection.")
try:
    cp = CommandProtocol.CommandProtocol(pc)

    applogger.info("Fetching all contacts from phone.")
    pBar = None
    phoneContactsToDelete = PhoneDOM.readAllPhoneMemoryContacts(cp, callback)
    applogger.info("Fetched all "+str(len(phoneContactsToDelete)) + " contacts from phone.")
    applogger.info("Fetching all contacts from Outlook.")    
    birthdays, outlookContacts = fetchBirthdaysContactsFromOutlook()
    applogger.info("Fetched all "+str(len(outlookContacts)) + " contacts from Outlook.")
    outlookContactsToUpload = PhoneDOM.PhoneItemDict()
    applogger.info("Matching up contacts.")
    resetProgressBar()
    count = 0
    for outlookContact in outlookContacts.values():
        keyOfEquivContact = findEquivalent(outlookContact, phoneContactsToDelete)
        if keyOfEquivContact:
            # No need to delete from phone. No need to send.
            del phoneContactsToDelete[keyOfEquivContact]
        else:
            # Need to send.
            outlookContactsToUpload.add(outlookContact)
        count = count + 1
        callback(count,len(outlookContacts),"Searched for match")
    applogger.info("Matched up contacts.")       
    applogger.info("Deleting "+str(len(phoneContactsToDelete)) + " unmatched contacts from phone")
    resetProgressBar()    
    phoneContactsToDelete.deleteFromPhone(cp, callback)
    applogger.info("Deleted unmatched contacts from phone")
    applogger.info("Uploading "+str(len(outlookContactsToUpload)) + " new contacts to phone")
    resetProgressBar()
    outlookContactsToUpload.writeToPhone(cp, callback)
    applogger.info("Uploaded new contacts to phone")


    applogger.info("Fetching all events from phone.")
    resetProgressBar()
    eventsToDelete = PhoneDOM.readAllEvents(cp, callback)
    applogger.info("Fetched all "+str(len(eventsToDelete)) + " events from phone.")

    maxEvents = max(0,400 - 40 - len(birthdays))
    # Phone can handle 400 events.
    # Leave 40 free to manually create.
    # Take off the birthdays
    
    pBar = None
    applogger.info("Fetching up to " + str(maxEvents) + " events from Outlook.")
    outlookEvents = fetchEventsFromOutlook(maxEvents)
    applogger.info("Fetched "+str(len(outlookEvents))+" events from Outlook")
    
    outlookEventsToUpload = PhoneDOM.PhoneItemDict()
    applogger.info("Matching up events.")
    resetProgressBar()
    count = 0
    for outlookEvent in outlookEvents.values():
        assert isinstance(outlookEvent,PhoneDOM.PhoneItem),str(outlookEvent)
        keyOfEquiv = findEquivalent(outlookEvent, eventsToDelete)
        if keyOfEquiv:
            # No need to delete from phone. No need to send.
            del eventsToDelete[keyOfEquiv]
        else:
            # Need to send.
            outlookEventsToUpload.add(outlookEvent)
        count = count + 1
        callback(count,len(outlookEvents),"Event Deleted")
    applogger.info("Matched up events.")
    applogger.info("Deleting "+str(len(eventsToDelete)) + " unmatched events from phone")
    resetProgressBar()
    eventsToDelete.deleteFromPhone(cp, callback)
    applogger.info("Deleted unmatched events from phone")
    applogger.info("Uploading "+str(len(outlookEventsToUpload)) + " new events to phone")
    resetProgressBar()
    outlookEventsToUpload.writeToPhone(cp, callback)
    applogger.info("Uploaded new events to phone")
    
finally:
    pc.close()
